package com.zfire.jiasm.syncdata.download.order.install;


import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.zfire.jiasm.syncdata.base.BaseService;
import com.zfire.jiasm.syncdata.constant.DictTypeEnum;
import com.zfire.jiasm.syncdata.constant.DispatchStatusEnum;
import com.zfire.jiasm.syncdata.constant.OrderStatusEnum;
import com.zfire.jiasm.syncdata.dao.mapper.AZGDFYQDDownloadMapper;
import com.zfire.jiasm.syncdata.dao.mapper.AZGDFkMxDownloadMapper;
import com.zfire.jiasm.syncdata.dao.mapper.AZGDLcLsDownloadMapper;
import com.zfire.jiasm.syncdata.parameter.OrderPar;
import com.zfire.jiasm.syncdata.plus.entity.*;
import com.zfire.jiasm.syncdata.plus.service.*;
import com.zfire.jiasm.syncdata.service.OrderService;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.BeanUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 格力品牌安装工单下载后落本地数据表
 * 3：落业务表
 *:4：修改下载主表处理结果
 */
@Slf4j
@Component
public class InstallBusiness {

    @Resource
    InstallBusiness installBusiness;
    @Resource
    ItfTblAzAssignLcLsDownloadService itfTblAzAssignLcLsDownloadService;
    @Resource
    ItfTblAzAssignMxDownloadService itfTblAzAssignMxDownloadService;
    @Resource
    AZGDLcLsDownloadMapper azGDLcLsDownloadMapper;
    @Resource
    BaseService baseService;
    @Resource
    WebsitOrderSignService websitOrderSignService;
    @Resource
    CustomerInformationWebsitService customerInformationWebsitService;
    @Resource
    OrderBaseService orderBaseService;
    @Resource
    OrderWorkerService orderWorkerService;
    @Resource
    SysDictRefService sysDictRefService;
    @Resource
    OrderProductService orderProductService;
    @Resource
    ItfTblAzAssignFkmxDownloadService itfTblAzAssignFkmxDownloadService;
    @Resource
    OrderInstallOperatingLogService orderInstallOperatingLogService;
    @Resource
    AZGDFkMxDownloadMapper azGDFkMxDownloadMapper;
    @Resource
    ItfTblAzAssignFeeDownloadService itfTblAzAssignFeeDownloadService;
    @Resource
    AZGDFYQDDownloadMapper azGDFYQDDownloadMapper;
    @Resource
    OrderInstallService orderInstallService;
    @Resource
    ItfTblAzAssignAppointmentDownloadService itfTblAzAssignAppointmentDownloadService;
    @Resource
    OrderAppointmentService orderAppointmentService;
    @Resource
    ItfTblAzAssignSatisfactionDownloadService itfTblAzAssignSatisfactionDownloadService;
    @Resource
    WebsitService websitService;
    @Resource
    OrderAppraiseService orderAppraiseService;
    @Resource
    OrderService orderService;



    // 3：工单落入本地表
    @Scheduled(fixedDelay = 1 * 10 * 1000)
    public void inputOrderInstall() {
        if (1 == 1) {
            return;
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<ItfTblAzAssignLcLsDownload> lcls = new ArrayList<>();
        List<String> lclslist = new ArrayList<>();
        List<OrderBase> orderBasesRecs = new ArrayList<>();
        List<String> orderBaselist = new ArrayList<>();
        Map<String, List<SysDictRef>> sysDictRefDLMap = new HashMap<>();
        Map<String, List<SysDictRef>> sysDictRefXLMap = new HashMap<>();
        Map<String, List<SysDictRef>> sysDictRefXiLieMap = new HashMap<>();

        // 提取品牌信息
        Brand brand = baseService.getBrand();
        if (brand == null) {
            return;
        }

        // 提取品牌商对应的大类信息
        sysDictRefDLMap = getStringListMap(brand);
        // 提取品牌商对应的小类信息
        sysDictRefXLMap = getListMap(brand);
        // 提取品牌商对应的系列信息
        sysDictRefXiLieMap = getMap(brand);

        // 提取待落本地表的数据
        lcls = itfTblAzAssignLcLsDownloadService.lambdaQuery()
                .eq(ItfTblAzAssignLcLsDownload::getSynStatus,0)
                .orderByAsc(ItfTblAzAssignLcLsDownload::getSynReqId)
                .list();

        if (lcls.size()<1){
            log.info("暂无待处理的安装工单: {}",sdf.format(new Date()));
            return;
        }

        log.info("安装工单落本地表开始处理: {}",sdf.format(new Date()));

        lclslist = lcls.stream().map(ItfTblAzAssignLcLsDownload::getPgguid).collect(Collectors.toList());
        // 提取已有的订单数据
        orderBasesRecs = orderBaseService.lambdaQuery()
                .select(OrderBase::getPgguid)
                .in(OrderBase::getPgguid,lclslist).list();
        orderBaselist = orderBasesRecs.stream().map(OrderBase::getPgguid).collect(Collectors.toList());


        // 按任务号download_task_no，按单来处理
        for (ItfTblAzAssignLcLsDownload item: lcls) {
            try{
                if (orderBaselist.contains(item.getPgguid())){
                    // 修改
                    installBusiness.orderUpdate(sysDictRefDLMap, sysDictRefXLMap, sysDictRefXiLieMap, brand, item);
                } else {
                    // 新增
                    installBusiness.orderAdd(sysDictRefDLMap, sysDictRefXLMap, sysDictRefXiLieMap, brand, item);
                    orderBaselist.add(item.getPgguid());
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                log.info("安装工单落本地表失败 销售单号-派工单号：【" + item.getXsdh() + "-"+ item.getPgid()+"】");
            }
        }

        log.info("安装工单落本地表完成: {}",sdf.format(new Date()));
    }

    @Transactional(rollbackFor = Exception.class)
    public void orderUpdate(Map<String, List<SysDictRef>> sysDictRefDLMap, Map<String, List<SysDictRef>> sysDictRefXLMap, Map<String, List<SysDictRef>> sysDictRefXiLieMap, Brand brand, ItfTblAzAssignLcLsDownload item) {
        List<Long> synReqYfYyIds = new ArrayList<>();
        List<OrderAppointment> addoalists = new ArrayList<>();
        List<ItfTblAzAssignMxDownload> mxsynreqids = new ArrayList<>();
        List<ItfTblAzAssignFkmxDownload> synreqids = new ArrayList<>();
        List<OrderProduct> orderProductsUpdate = new ArrayList<>();
        List<OrderInstallOperatingLog> addoiolList = new ArrayList<>();
        List<Long> getpjSynReqId = new ArrayList<>();
        List<OrderAppraise> addoalist = new ArrayList<>();

        // 提取本地订单
        OrderBase old = orderBaseService.lambdaQuery()
                .eq(OrderBase::getPgguid, item.getPgguid()).one();

        // 获取安装工单主表
        OrderBase newLCLS = getOrderBase(item, brand);
        // 这里一定不能少
        newLCLS.setId(old.getId());
        newLCLS.setDispatchStatus(old.getDispatchStatus());

        // 获取工单主表要同步的数据
        OrderBase upInfo = getOrderBase(old, item, newLCLS);

        // 获取工单师傅信息来自于工单主表数据只有修改的情况
        OrderWorker own =getOrderWorkerUpdate(old.getId(), upInfo, newLCLS);

        // 获取产品明细要同步的数据 -只存在修改，减少与增加的情况不存在
        orderProductsUpdate = getOrderProductsUpdate(sysDictRefDLMap, sysDictRefXLMap, sysDictRefXiLieMap, old, newLCLS, item.getDownloadTaskNo(),mxsynreqids);

        // 获取操作明细同步数据，即反馈明细
        addoiolList = getOrderInstallOperatingLogsadd(brand, item, old, newLCLS, synreqids);

        // 费用清单 放扩展表
        // 提取已同步的数据
        OrderInstall oldOrderInstall = orderInstallService.lambdaQuery()
                .eq(OrderInstall::getOrderBaseId,old.getId()).one();
        // 获取安装费用清单
        ItfTblAzAssignFeeDownload fyqd = itfTblAzAssignFeeDownloadService.lambdaQuery()
                .eq(ItfTblAzAssignFeeDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAzAssignFeeDownload::getSynStatus,0)
                .one();
        OrderInstall orderInstall = getOrderInstall(newLCLS, brand, item, fyqd);
        // 已同步的有数据，后期的一定有数据  直接修改
        if (oldOrderInstall != null){
            orderInstall.setId(oldOrderInstall.getId());
            orderInstall.setUpdateBy(newLCLS.getLastOperatorBy());
            orderInstall.setUpdateTime(newLCLS.getLastOperatorTime());
        }

        // 获取用户预约改约信息
        synReqYfYyIds = getYfYyLongs(item, old, newLCLS, addoalists);

        // 获取评价同步数据
        addoalist = getOrderAppraises(sysDictRefDLMap, brand, item, old, newLCLS, getpjSynReqId);

        // 安装工单数据修改落本地表
        addOrUpdateOrderData(item, upInfo, own, mxsynreqids, orderProductsUpdate, synreqids,
                addoiolList, fyqd, orderInstall, synReqYfYyIds, addoalists, getpjSynReqId, addoalist);
    }

    public void addOrUpdateOrderData(ItfTblAzAssignLcLsDownload item, OrderBase upInfo, OrderWorker own,
                                     List<ItfTblAzAssignMxDownload> mxsynreqids, List<OrderProduct> orderProductsUpdate,
                                     List<ItfTblAzAssignFkmxDownload> synreqids, List<OrderInstallOperatingLog> addoiolList,
                                     ItfTblAzAssignFeeDownload fyqd, OrderInstall orderInstall, List<Long> synReqYfYyIds,
                                     List<OrderAppointment> addoalists, List<Long> getpjSynReqId, List<OrderAppraise> addoalist) {
        // 同步修改订单基础表
        if (upInfo !=null){
            orderBaseService.updateById(upInfo);
            // lc_ls同步状态改成1 同步成功
            synItfTblAzAssignLcLsDownload(item.getSynReqId(),1,"成功");
        }

        // 同步新增或修改师傅信息 own 从工单主表过来，无load表修改
        if (own != null){
            orderWorkerService.saveOrUpdate(own);
        }

        // 同步修改产品明细表
        if (orderProductsUpdate.size()>0){
            orderProductService.updateBatchById(orderProductsUpdate);
            orderProductsUpdate.clear();
        }
        if (mxsynreqids.size()>0){
            itfTblAzAssignMxDownloadService.updateBatchById(mxsynreqids);
            mxsynreqids.clear();
        }

        // 同步新增操作日志表
        if (addoiolList.size()>0){
            orderInstallOperatingLogService.saveBatch(addoiolList);
            addoiolList.clear();
        }
        if (synreqids.size()>0){
            itfTblAzAssignFkmxDownloadService.updateBatchById(synreqids);
            synreqids.clear();
        }

        // 同步修改安装工单扩展表
        if (orderInstall !=null){
            orderInstallService.saveOrUpdate(orderInstall);
        }
        if (fyqd != null){
            upItfTblAzAssignFeeDownload(fyqd);
        }

        // 同步用户预约表
        if (addoalists.size()>0){
            orderAppointmentService.saveBatch(addoalists);
            addoalist.clear();
        }
        if (synReqYfYyIds.size()>0){
            upItfTblAzAssignAppointment(synReqYfYyIds);
            synReqYfYyIds.clear();
        }

        // 同步评价新增的数据
        if (addoalist.size()>0){
            orderAppraiseService.saveBatch(addoalist);
            addoalist.clear();
        }
        if (getpjSynReqId.size()>0){
            upItfTblAzAssignSatisfactionDownload(getpjSynReqId);
            getpjSynReqId.clear();
        }
    }

    @NotNull
    private List<Long> getYfYyLongs(ItfTblAzAssignLcLsDownload item, OrderBase old, OrderBase newLCLS, List<OrderAppointment> addoalists) {
        List<Long> synReqYfYyIds;
        List<OrderAppointment> oaps = new ArrayList<>();
        // 提取已同步的
        List<OrderAppointment> oapps = orderAppointmentService.lambdaQuery()
                .select(OrderAppointment::getId)
                .eq(OrderAppointment::getOrderBaseId, old.getId()).list();
        List<String> oappslist  = oapps.stream().map(OrderAppointment::getId).collect(Collectors.toList());

        // 获取用户预约改约信息
        List<ItfTblAzAssignAppointmentDownload> appointments = itfTblAzAssignAppointmentDownloadService.lambdaQuery()
                .eq(ItfTblAzAssignAppointmentDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAzAssignAppointmentDownload::getSynStatus,0)
                .orderByAsc(ItfTblAzAssignAppointmentDownload::getSynReqId)
                .list();
        synReqYfYyIds = appointments.stream().map(ItfTblAzAssignAppointmentDownload::getSynReqId).collect(Collectors.toList());
        // 提取本次获取的
        getOrderAppointment(newLCLS, oaps, appointments);

        // 移除已同步过的
        for (OrderAppointment oa:oaps){
            if (!oappslist.contains(oa.getId())){
                addoalists.add(oa);
            }
        }
        return synReqYfYyIds;
    }

    @NotNull
    private List<OrderAppraise> getOrderAppraises(Map<String, List<SysDictRef>> sysDictRefDLMap, Brand brand, ItfTblAzAssignLcLsDownload item, OrderBase old, OrderBase newLCLS, List<Long> getpjSynReqId) {
        // 提取已同步的
        List<OrderAppraise> oas = orderAppraiseService.lambdaQuery()
                .select(OrderAppraise::getPid,
                        OrderAppraise::getPgguid)
                .eq(OrderAppraise::getOrderBaseId, old.getId()).list();
        List<String> oaslist  = oas.stream().map(OrderAppraise::getPid).collect(Collectors.toList());

        // 提取本次获取的
        List<OrderAppraise> oanew = new ArrayList<>();
        getOrderAppraise(newLCLS, sysDictRefDLMap, oanew, brand, item, getpjSynReqId);

        // 移除已同步过的
        List<OrderAppraise> addoalist = new ArrayList<>();
        for (OrderAppraise oa:oanew){
            if (!oaslist.contains(oa.getPid())){
                addoalist.add(oa);
            }
        }
        return addoalist;
    }

    private void upItfTblAzAssignSatisfactionDownload(List<Long> getpjSynReqId) {
        List<ItfTblAzAssignSatisfactionDownload> sas =new ArrayList<>();
        for (int i = 0; i< getpjSynReqId.size(); i++){
            ItfTblAzAssignSatisfactionDownload add = new ItfTblAzAssignSatisfactionDownload();
            add.setSynReqId(getpjSynReqId.get(i));
            add.setSynStatus(1);
            add.setSynTime(new Date());
            add.setSynErrMsg("成功");
            sas.add(add);
        }
        itfTblAzAssignSatisfactionDownloadService.updateBatchById(sas);
    }

    @NotNull
    private List<OrderInstallOperatingLog> getOrderInstallOperatingLogsadd(Brand brand, ItfTblAzAssignLcLsDownload item, OrderBase old,
                                                                           OrderBase newLCLS, List<ItfTblAzAssignFkmxDownload> synreqids) {
        // 提取已同步的
        List<OrderInstallOperatingLog> oiols = orderInstallOperatingLogService.lambdaQuery()
                .select(OrderInstallOperatingLog::getPgguid,
                        OrderInstallOperatingLog::getFkid)
                .eq(OrderInstallOperatingLog::getOrderBaseId, old.getId()).list();

        List<String> oiolslist  = oiols.stream().map(OrderInstallOperatingLog::getFkid).collect(Collectors.toList());

        List<ItfTblAzAssignFkmxDownload> fkmxs = itfTblAzAssignFkmxDownloadService.lambdaQuery()
                .eq(ItfTblAzAssignFkmxDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAzAssignFkmxDownload::getSynStatus,0)
                .orderByAsc(ItfTblAzAssignFkmxDownload::getCreatedDate)
                .list();

        List<OrderInstallOperatingLog> oiolList = getOrderInstallOperatingLogs(fkmxs, newLCLS, brand, item);
        // 移除已同步过的
        List<OrderInstallOperatingLog> addoiolList = new ArrayList<>();
        for (OrderInstallOperatingLog ol:oiolList){
            if (!oiolslist.contains(ol.getFkid())){
                addoiolList.add(ol);
            }
        }

        for (ItfTblAzAssignFkmxDownload i : fkmxs){
            ItfTblAzAssignFkmxDownload n = new ItfTblAzAssignFkmxDownload();
            n.setSynReqId(i.getSynReqId());
            n.setSynStatus(1);
            n.setSynTime(new Date());
            synreqids.add(n);
        }

        return addoiolList;
    }

    private OrderWorker getOrderWorkerUpdate(String id, OrderBase upInfo, OrderBase newLCLS){
        OrderWorker oow = orderWorkerService.lambdaQuery()
                .eq(OrderWorker::getOrderBaseId,id)
                .eq(OrderWorker::getOutWorkerSource,Boolean.FALSE).one();
        OrderWorker own = new OrderWorker();
        Boolean uf = Boolean.FALSE;

        if (upInfo !=null){
            OrderWorker newow = getOrderWorker(newLCLS);
            if (oow ==null){
                if (newow !=null){
                    return newow;
                }
                return null;
            }

            // 师傅信息变更，或是改派师傅
            if (!oow.getWorkerId().equals(newow.getWorkerId())) {
                own.setWorkerId(newow.getWorkerId());
                uf = Boolean.TRUE;
            }
            if (!oow.getWorkerName().equals(newow.getWorkerName())) {
                own.setWorkerName(newow.getWorkerName());
                uf = Boolean.TRUE;
            }
            if (!oow.getWorkerMobile().equals(newow.getWorkerMobile())) {
                own.setWorkerMobile(newow.getWorkerMobile());
                uf = Boolean.TRUE;
            }
            if (!oow.getWorkerIdcard().equals(newow.getWorkerIdcard())) {
                own.setWorkerIdcard(newow.getWorkerIdcard());
                uf = Boolean.TRUE;
            }
            if (uf){
                own.setId(oow.getId());
                own.setUpdateBy(newLCLS.getLastOperatorBy());
                own.setUpdateTime(newLCLS.getLastOperatorTime());
                return own;
            }
        }
        return null;
    }

    private List<OrderProduct> getOrderProductsUpdate(Map<String, List<SysDictRef>> sysDictRefDLMap, Map<String,
            List<SysDictRef>> sysDictRefXLMap, Map<String, List<SysDictRef>> sysDictRefXiLieMap,
                                                      OrderBase old, OrderBase newLCLS, String downloadTaskNo,
                                                      List<ItfTblAzAssignMxDownload> synreqids) {
        List<OrderProduct> orderProductsNew = new ArrayList<>();
        List<OrderProduct> orderProductsOld = new ArrayList<>();
        List<OrderProduct> orderProductsUpdate = new ArrayList<>();
        Boolean uf = Boolean.FALSE;

        // 提取下载产品明细表
        List<ItfTblAzAssignMxDownload> orderMx = itfTblAzAssignMxDownloadService.lambdaQuery()
                .eq(ItfTblAzAssignMxDownload::getDownloadTaskNo, downloadTaskNo)
                .eq(ItfTblAzAssignMxDownload::getSynStatus,0)
                .orderByAsc(ItfTblAzAssignMxDownload::getSynReqId)
                .list();

        // 获取新同步的工单机型
        getOrderProduct(newLCLS, orderProductsNew, sysDictRefDLMap, sysDictRefXLMap, sysDictRefXiLieMap, orderMx);
        // 产品只存在修改，减少与增加的情况不存在
        // 提取本地现有的产品机型
        orderProductsOld = orderProductService.lambdaQuery()
                .eq(OrderProduct::getOrderBaseId, old.getId()).list();
        for (OrderProduct o : orderProductsOld){
            for (OrderProduct n :orderProductsNew){
                if (n.getPgmxid().equals(o.getPgmxid())){
                    uf = Boolean.FALSE;
                    OrderProduct v = new OrderProduct();
                    // 查看有无要修改的机型信息
                    if (!o.getMainId().equals(n.getMainId())) {
                        v.setMainId(n.getMainId());
                        v.setMainName(n.getMainName());
                        uf = Boolean.TRUE;
                    }
                    if (!o.getSmallId().equals(n.getSmallId())) {
                        v.setSmallId(n.getSmallId());
                        v.setSmallName(n.getSmallName());
                        uf = Boolean.TRUE;
                    }
                    // 系列可不填写
                    if (StringUtils.isNotEmpty(o.getSeriesId()) && StringUtils.isNotEmpty(n.getSeriesId())) {
                        if (!o.getSeriesId().equals(n.getSeriesId())){
                            v.setSeriesId(n.getSeriesId());
                            v.setSeriesName(n.getSeriesName());
                            uf = Boolean.TRUE;
                        }
                    } else {
                        if (StringUtils.isNotEmpty(o.getSeriesId())) {
                            v.setSeriesId("");
                            v.setSeriesName("");
                        } else {
                            v.setSeriesId(n.getSeriesId());
                            v.setSeriesName(n.getSeriesName());
                        }
                        uf = Boolean.TRUE;
                    }
                    if (!o.getProductId().equals(n.getProductId())) {
                        v.setProductId(n.getProductId());
                        v.setProductName(n.getProductName());
                        uf = Boolean.TRUE;
                    }
                    if (StringUtils.isNotEmpty(o.getInsideCode()) && StringUtils.isNotEmpty(n.getInsideCode())) {
                        if (!o.getInsideCode().equals(n.getInsideCode())){
                            v.setInsideCode(n.getInsideCode());
                            uf = Boolean.TRUE;
                        }
                    } else {
                        if (StringUtils.isNotEmpty(o.getInsideCode())) {
                            v.setInsideCode("");
                        } else {
                            v.setInsideCode(n.getInsideCode());
                        }
                        uf = Boolean.TRUE;
                    }
                    if (StringUtils.isNotEmpty(o.getOutCode()) && StringUtils.isNotEmpty(n.getOutCode())) {
                        if (!o.getOutCode().equals(n.getOutCode())){
                            v.setOutCode(n.getOutCode());
                            uf = Boolean.TRUE;
                        }
                    } else {
                        if (StringUtils.isNotEmpty(o.getOutCode())) {
                            v.setOutCode("");
                        } else {
                            v.setOutCode(n.getOutCode());
                        }
                        uf = Boolean.TRUE;
                    }
                    if (o.getNum() !=null && n.getNum() !=null) {
                        if (!o.getNum().equals(n.getNum())){
                            v.setNum(n.getNum());
                            uf = Boolean.TRUE;
                        }
                    } else {
                        if (o.getNum() !=null) {
                            v.setNum(null);
                        } else {
                            v.setNum(n.getNum());
                        }
                        uf = Boolean.TRUE;
                    }
                    if (o.getPrice() !=null && n.getPrice() !=null) {
                        if (!o.getPrice().equals(n.getPrice())){
                            v.setPrice(n.getPrice());
                            uf = Boolean.TRUE;
                        }
                    } else {
                        if (o.getPrice() !=null) {
                            v.setPrice(null);
                        } else {
                            v.setPrice(n.getPrice());
                        }
                        uf = Boolean.TRUE;
                    }
                    if (StringUtils.isNotEmpty(o.getUnit()) && StringUtils.isNotEmpty(n.getUnit())) {
                        if (!o.getUnit().equals(n.getUnit())){
                            v.setUnit(n.getUnit());
                            uf = Boolean.TRUE;
                        }
                    } else {
                        if (StringUtils.isNotEmpty(o.getUnit())) {
                            v.setUnit("");
                        } else {
                            v.setUnit(n.getUnit());
                        }
                        uf = Boolean.TRUE;
                    }
                    if (StringUtils.isNotEmpty(o.getMaterialNumber()) && StringUtils.isNotEmpty(n.getMaterialNumber())) {
                        if (!o.getMaterialNumber().equals(n.getMaterialNumber())){
                            v.setMaterialNumber(n.getMaterialNumber());
                            uf = Boolean.TRUE;
                        }
                    } else {
                        if (StringUtils.isNotEmpty(o.getMaterialNumber())) {
                            v.setMaterialNumber("");
                        } else {
                            v.setMaterialNumber(n.getUnit());
                        }
                        uf = Boolean.TRUE;
                    }
                    if (!o.getInsideNum().equals(n.getInsideNum())) {
                        v.setInsideNum(n.getInsideNum());
                        uf = Boolean.TRUE;
                    }
                    if (!o.getOutNum().equals(n.getOutNum())) {
                        v.setOutNum(n.getOutNum());
                        uf = Boolean.TRUE;
                    }
                    if (!o.getUndoneNum().equals(n.getUndoneNum())) {
                        v.setUndoneNum(n.getUndoneNum());
                        uf = Boolean.TRUE;
                    }
                    if (StringUtils.isNotEmpty(o.getSaleWebsit()) && StringUtils.isNotEmpty(n.getSaleWebsit())) {
                        if (!o.getSaleWebsit().equals(n.getSaleWebsit())){
                            v.setSaleWebsit(n.getSaleWebsit());
                            uf = Boolean.TRUE;
                        }
                    } else {
                        if (StringUtils.isNotEmpty(o.getSaleWebsit())) {
                            v.setSaleWebsit("");
                        } else {
                            v.setSaleWebsit(n.getSaleWebsit());
                        }
                        uf = Boolean.TRUE;
                    }
                    if (StringUtils.isNotEmpty(o.getServiceWebsit()) && StringUtils.isNotEmpty(n.getServiceWebsit())) {
                        if (!o.getServiceWebsit().equals(n.getServiceWebsit())){
                            v.setServiceWebsit(n.getServiceWebsit());
                            uf = Boolean.TRUE;
                        }
                    } else {
                        if (StringUtils.isNotEmpty(o.getServiceWebsit())) {
                            v.setServiceWebsit("");
                        } else {
                            v.setServiceWebsit(n.getServiceWebsit());
                        }
                        uf = Boolean.TRUE;
                    }

                    if (uf){
                        v.setId(o.getId());
                        v.setUpdateBy(newLCLS.getLastOperatorBy());
                        v.setUpdateTime(newLCLS.getLastOperatorTime());
                        orderProductsUpdate.add(o);
                    }
                }
            }
        }
        for (ItfTblAzAssignMxDownload i : orderMx){
            ItfTblAzAssignMxDownload n = new ItfTblAzAssignMxDownload();
            n.setSynReqId(i.getSynReqId());
            n.setSynStatus(1);
            n.setSynTime(new Date());
            synreqids.add(n);
        }

        return orderProductsUpdate;
    }

    @NotNull
    private OrderBase getOrderBase(OrderBase old, ItfTblAzAssignLcLsDownload item, OrderBase newLCLS) {
        OrderBase upInfo = new OrderBase();

        // 本地订单已关闭 且总部的最后修改时间晚于本地数据修改时间 不可修改
        if (old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I10.getCode()) &&
                item.getLastModifiedDate().compareTo(old.getUpdateTime())<0){
            synItfTblAzAssignLcLsDownload(item.getSynReqId(), 99, "本地工单已关闭或作废忽略更新");
            // 这里只是不修改订单主表，其他数据表接着执行
            return null;
        }

        // 1.安装工单主表基础信息更新
        if (item.getLastModifiedDate().compareTo(old.getUpdateTime())>0){
            BeanUtils.copyProperties(newLCLS,upInfo);
        }
        // 2:网点信息修改 已待服务人员处理，不允更新网点   6,7,10 少6
        if (old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I6.getCode()) ||
                old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I10.getCode()) ||
                old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I7.getCode())){
            // 售后网点
            upInfo.setReceWebsitId(old.getReceWebsitId());
            upInfo.setReceWebsitName(old.getReceWebsitName());
            // 安装
            upInfo.setWebsitId(old.getWebsitId());
            upInfo.setWebsitName(old.getWebsitName());
        }
        // 3:已预约不可更改预约时间
        if (old.getFirstAppointmentTime() !=null){
            upInfo.setAppointmentTime(old.getAppointmentTime());
        }

        OrderInstall oi = orderInstallService.lambdaQuery()
                .eq(OrderInstall::getOrderBaseId,old.getId()).one();

        // 3.2天猫工单，允许天猫自已预约
        if (oi !=null && StringUtils.isNotEmpty(oi.getExtendField2()) &&
                oi.getExtendField2().equals("TBL_TMALL_ORDERINFO")){
            upInfo.setAppointmentTime(newLCLS.getAppointmentTime());
        }

        // 4:工单状态
        Boolean allowUpdate = getAllowUpdate(old, newLCLS);
        if (allowUpdate && item.getLastModifiedDate().compareTo(old.getUpdateTime())>0){
            upInfo.setOrderStatus(newLCLS.getOrderStatus());
            upInfo.setOrderStatus(newLCLS.getOrderStatus());
            upInfo.setOrderStatus(newLCLS.getOrderStatus());
        }

        upInfo.setId(old.getId());
        upInfo.setOutUpdateSynReqId(item.getSynReqId());
        upInfo.setOutUpdateSynReqTime(new Date());
        upInfo.setUpdateTime(new Date());
        return upInfo;
    }


    private void synItfTblAzAssignLcLsDownload(Long synReqId, Integer synStatus, String msg){
        ItfTblAzAssignLcLsDownload uplcls = new ItfTblAzAssignLcLsDownload();
        uplcls.setSynReqId(synReqId);
        uplcls.setSynStatus(synStatus);
        uplcls.setSynTime(new Date());
        uplcls.setSynErrMsg(msg);
        itfTblAzAssignLcLsDownloadService.updateById(uplcls);
    }


    @NotNull
    private Boolean getAllowUpdate(OrderBase old, OrderBase newLCLS) {
        // 判断是否可更新工单状态
        Boolean allowUpdate = Boolean.FALSE;
        // 待派
        if ((old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I1.getCode()) ||
                old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I2.getCode()) ||
                old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I3.getCode())) &&
                (old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_0.getCode()))){
            allowUpdate = Boolean.TRUE;
        }
        // 改派
        if ((old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I3.getCode())) &&
                (old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_0.getCode())) &&
                (newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I3.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I4.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I6.getCode()))){
            allowUpdate = Boolean.TRUE;
        }
        // 待接单
        if ((old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I4.getCode())) &&
                (old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_0.getCode())) &&
                (newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I7.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I8.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I9.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I10.getCode()))){
            allowUpdate = Boolean.TRUE;
        }
        // 已接单
        if ((old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I6.getCode())) &&
                (old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_0.getCode()) ||
                        old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_1.getCode())||
                        old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_2.getCode()) ||
                        old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_3.getCode())) &&
                (newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I7.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I8.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I9.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I10.getCode()))){
            allowUpdate = Boolean.TRUE;
        }
        // 服务人员完工
        if ((old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I7.getCode())) &&
                (old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_4.getCode()) ||
                        old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_1.getCode())||
                        old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_2.getCode()) ||
                        old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_3.getCode())) &&
                (newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I8.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I9.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I10.getCode()))){
            allowUpdate = Boolean.TRUE;
        }
        // 用户改约
        if ((old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I5.getCode())) &&
                (old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_5.getCode())) &&
                (newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I8.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I9.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I10.getCode()))){
            allowUpdate = Boolean.TRUE;
        }
        // 作废申请
        if ((old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I2.getCode())) &&
                (old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_2.getCode())) &&
                (newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I3.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I4.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I6.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I9.getCode()))){
            allowUpdate = Boolean.TRUE;
        }

        // 广佛作废时如果总部下载1121或1129也同步状态
        if (old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I9.getCode()) ||
                old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I10.getCode())){
            if (newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I2.getCode()) ||
                    newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I3.getCode()) ||
                    newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I4.getCode())){
                allowUpdate = Boolean.TRUE;
            } else {
                allowUpdate = Boolean.FALSE;
            }
        } else {
            allowUpdate = Boolean.TRUE;
        }
        if ((old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I9.getCode()) ||
                old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I10.getCode())) &&
                (old.getDispatchStatus().equals(DispatchStatusEnum.INSTALL_DISPATCH_STATUS_2.getCode())) &&
                (newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I3.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I4.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I6.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I9.getCode()))){
            allowUpdate = Boolean.TRUE;
        }

        // 广佛已自动派工，不根据总部状态更新
        if ((old.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I6.getCode())) &&
                (newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I2.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I3.getCode()) ||
                        newLCLS.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I6.getCode()))){
            allowUpdate = Boolean.FALSE;
        }
        return allowUpdate;
    }

    @Transactional(rollbackFor = Exception.class)
    public void orderAdd(Map<String, List<SysDictRef>> sysDictRefDLMap, Map<String, List<SysDictRef>> sysDictRefXLMap,
                          Map<String, List<SysDictRef>> sysDictRefXiLieMap, Brand brand, ItfTblAzAssignLcLsDownload item) {
        List<OrderAppointment> oa = new ArrayList<>();
        List<OrderAppraise> oanew = new ArrayList<>();
        List<OrderProduct> orderProducts = new ArrayList<>();
        List<ItfTblAzAssignFkmxDownload> fkmxs = new ArrayList<>();
        List<Long> synReqIds = new ArrayList<>();
        List<Long> synReqFkMxIds = new ArrayList<>();
        List<Long> synReqYfYyIds = new ArrayList<>();

        List<ItfTblAzAssignMxDownload> orderMx = itfTblAzAssignMxDownloadService.lambdaQuery()
                .eq(ItfTblAzAssignMxDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAzAssignMxDownload::getSynStatus,0)
                .orderByAsc(ItfTblAzAssignMxDownload::getSynReqId)
                .list();
        synReqIds = orderMx.stream().map(ItfTblAzAssignMxDownload::getSynReqId).collect(Collectors.toList());

        fkmxs = itfTblAzAssignFkmxDownloadService.lambdaQuery()
                .eq(ItfTblAzAssignFkmxDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAzAssignFkmxDownload::getSynStatus,0)
                .orderByAsc(ItfTblAzAssignFkmxDownload::getCreatedDate)
                .list();
        synReqFkMxIds = fkmxs.stream().map(ItfTblAzAssignFkmxDownload::getSynReqId).collect(Collectors.toList());

        // 获取安装工单主表
        OrderBase orderBase = getOrderBase(item, brand);
        // 获取工单师傅信息
        OrderWorker orderWorker = getOrderWorker(orderBase);
        // 获取工单机型
        getOrderProduct(orderBase, orderProducts, sysDictRefDLMap, sysDictRefXLMap, sysDictRefXiLieMap, orderMx);
        // 获取工单操作明细- 操作日志表
        List<OrderInstallOperatingLog> oiolList = getOrderInstallOperatingLogs(fkmxs, orderBase, brand, item);
        // 获取安装费用清单 - 对应扩展表
        ItfTblAzAssignFeeDownload fyqd = itfTblAzAssignFeeDownloadService.lambdaQuery()
                .eq(ItfTblAzAssignFeeDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAzAssignFeeDownload::getSynStatus,0)
                .one();
        OrderInstall orderInstall = getOrderInstall(orderBase, brand, item, fyqd);
        // 获取用户预约改约信息
        List<ItfTblAzAssignAppointmentDownload> appointments = itfTblAzAssignAppointmentDownloadService.lambdaQuery()
                .eq(ItfTblAzAssignAppointmentDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAzAssignAppointmentDownload::getSynStatus,0)
                .orderByAsc(ItfTblAzAssignAppointmentDownload::getSynReqId)
                .list();
        synReqYfYyIds = appointments.stream().map(ItfTblAzAssignAppointmentDownload::getSynReqId).collect(Collectors.toList());
        getOrderAppointment(orderBase, oa, appointments);

        // 获取用户客户评价信息
        List<Long> getpjSynReqId = new ArrayList<>();
        getOrderAppraise(orderBase, sysDictRefDLMap, oanew, brand, item, getpjSynReqId);

        // 安装工单数据落本地表
        addOrderData(orderBase, orderWorker, orderInstall, orderProducts, oiolList, oa, oanew,
                item.getSynReqId(), synReqIds, synReqFkMxIds, synReqYfYyIds, fyqd, getpjSynReqId);
    }

    /**
     * 工单基础表order_base,工单师傅表order_worker,工单机型表order_product,
     * 操作明细表order_install_operating_log,安装费用清单表order_install,用户预约改约信息表order_appointment,
     * 工单评价表order_appraise
     */
    public void addOrderData(OrderBase orderBase, OrderWorker orderWorker, OrderInstall orderInstall, List<OrderProduct> orderProducts,
                             List<OrderInstallOperatingLog> oiolList, List<OrderAppointment> oa, List<OrderAppraise> oanew,
                             Long synReqId, List<Long> synReqIds,  List<Long> synReqFkMxIds, List<Long> synReqYfYyIds,
                             ItfTblAzAssignFeeDownload fyqd, List<Long> getpjSynReqId) {
        // 工单基础表
        if (orderBase != null) {
            orderBaseService.save(orderBase);
        }
        upItfTblAzAssignLcLsDownload(synReqId);

        // 工单师傅表
        if (orderWorker != null) {
            orderWorkerService.save(orderWorker);
        }

        // 工单机型表
        if (orderProducts.size()>0){
            orderProductService.saveBatch(orderProducts);
            orderProducts.clear();
        }
        if (synReqIds.size()>0){
            upItfTblAzAssignMxDownload(synReqIds);
            synReqIds.clear();
        }

        // 操作明细表
        if (oiolList.size()>0){
            orderInstallOperatingLogService.saveBatch(oiolList);
            oiolList.clear();
        }
        if (synReqFkMxIds.size()>0){
            upItfTblAzAssignFkmxDownload(synReqFkMxIds);
            synReqFkMxIds.clear();
        }

        // 安装费用清单表
        if (orderInstall != null){
            orderInstallService.save(orderInstall);
        }
        if (fyqd !=null){
            upItfTblAzAssignFeeDownload(fyqd);
        }

        // 用户预约改约信息表
        if (oa.size()>0){
            orderAppointmentService.saveBatch(oa);
            oa.clear();
        }
        if (synReqYfYyIds.size()>0){
            upItfTblAzAssignAppointment(synReqYfYyIds);
            synReqYfYyIds.clear();
        }

        // 工单评价表
        if (oanew.size()>0){
            orderAppraiseService.saveBatch(oanew);
            oanew.clear();
        }
        if (getpjSynReqId.size()>0){
            upItfTblAzAssignSatisfactionDownload(getpjSynReqId);
            getpjSynReqId.clear();
        }
    }

    private void upItfTblAzAssignFeeDownload(ItfTblAzAssignFeeDownload fyqd) {
        ItfTblAzAssignFeeDownload add = new ItfTblAzAssignFeeDownload();
        add.setSynReqId(fyqd.getSynReqId());
        add.setSynStatus(1);
        add.setSynTime(new Date());
        add.setSynErrMsg("成功");
        itfTblAzAssignFeeDownloadService.updateById(add);
    }

    private void upItfTblAzAssignMxDownload(List<Long> synReqIds) {
        List<ItfTblAzAssignMxDownload> sas =new ArrayList<>();
        for (int i = 0; i< synReqIds.size(); i++){
            ItfTblAzAssignMxDownload add = new ItfTblAzAssignMxDownload();
            add.setSynReqId(synReqIds.get(i));
            add.setSynStatus(1);
            add.setSynTime(new Date());
            add.setSynErrMsg("成功");
            sas.add(add);
        }
        itfTblAzAssignMxDownloadService.updateBatchById(sas);
        // 测试用
        /*if (synReqIds.get(0).equals(190L)){
            itfTblAzAssignMxDownloadService.updateBatchById(sas);
        } else {
            itfTblAzAssignMxDownloadService.saveBatch(sas);
        }*/
    }

    private void upItfTblAzAssignFkmxDownload(List<Long> synReqFkMxIds) {
        List<ItfTblAzAssignFkmxDownload> sas =new ArrayList<>();
        for (int i = 0; i< synReqFkMxIds.size(); i++){
            ItfTblAzAssignFkmxDownload add = new ItfTblAzAssignFkmxDownload();
            add.setSynReqId(synReqFkMxIds.get(i));
            add.setSynStatus(1);
            add.setSynTime(new Date());
            add.setSynErrMsg("成功");
            sas.add(add);
        }
        itfTblAzAssignFkmxDownloadService.updateBatchById(sas);
    }

    private void upItfTblAzAssignAppointment(List<Long> synReqYfYYIds) {
        List<ItfTblAzAssignAppointmentDownload> sas =new ArrayList<>();
        for (int i = 0; i< synReqYfYYIds.size(); i++){
            ItfTblAzAssignAppointmentDownload add = new ItfTblAzAssignAppointmentDownload();
            add.setSynReqId(synReqYfYYIds.get(i));
            add.setSynStatus(1);
            add.setSynTime(new Date());
            add.setSynErrMsg("成功");
            sas.add(add);
        }
        itfTblAzAssignAppointmentDownloadService.updateBatchById(sas);
    }

    private void upItfTblAzAssignLcLsDownload(Long synReqId) {
        ItfTblAzAssignLcLsDownload add = new ItfTblAzAssignLcLsDownload();
        add.setSynReqId(synReqId);
        add.setSynStatus(1);
        add.setSynTime(new Date());
        add.setSynErrMsg("成功");
        itfTblAzAssignLcLsDownloadService.updateById(add);
    }

    private void getOrderAppraise(OrderBase orderBase, Map<String, List<SysDictRef>> sysDictRefDLMap, List<OrderAppraise> oanew, Brand brand,
                                  ItfTblAzAssignLcLsDownload item, List<Long> getSynReqId) {
        List<ItfTblAzAssignSatisfactionDownload> sas;
        sas = itfTblAzAssignSatisfactionDownloadService.lambdaQuery()
                .eq(ItfTblAzAssignSatisfactionDownload::getDownloadTaskNo, item.getDownloadTaskNo())
                .eq(ItfTblAzAssignSatisfactionDownload::getSynStatus,0)
                .orderByAsc(ItfTblAzAssignSatisfactionDownload::getSynReqId)
                .list();
        for (ItfTblAzAssignSatisfactionDownload s :sas){
            getSynReqId.add(s.getSynReqId());
            OrderAppraise o = new OrderAppraise();
            o.setPid(s.getId());
            o.setPgguid(s.getPgguid());
            // 工单号
            o.setOrderBaseId(orderBase.getId());
            if (s.getPjly().equals("1")){
                o.setAppraiseStatus("A");
            } else if (s.getPjly().equals("2")){
                o.setAppraiseStatus("B");
            } else if (s.getPjly().equals("3")){
                o.setAppraiseStatus("C");
            } else {
                o.setAppraiseStatus("X");
            }
            // 回访人id无
            o.setRvisitNickName(s.getHfren());
            // 回访人网点信息

            // 先从售后网点查
            CustomerInformationWebsit ciw = null;
            WebsitOrderSign wos = websitOrderSignService.lambdaQuery()
                    .eq(WebsitOrderSign::getBrandWebsitId, s.getHfwdno())
                    .eq(WebsitOrderSign::getBrandId, brand.getBrandId())
                    .one();
            if (wos !=null){
                o.setRvisitWebsitId(wos.getWebsitId());
            } else {
                ciw = customerInformationWebsitService.lambdaQuery()
                        .eq(CustomerInformationWebsit::getRefWebsitId, s.getHfwdno())
                        .eq(CustomerInformationWebsit::getBrandId, brand.getBrandId())
                        .one();
                if (ciw !=null){
                    o.setRvisitWebsitId(wos.getWebsitId());
                }
            }
            if (wos !=null || ciw !=null){
                Websit websit = websitService.lambdaQuery()
                        .eq(Websit::getWebsitId,o.getRvisitWebsitId())
                        .one();
                o.setRvisitWebsitName(websit.getWebsitName());
            }
            o.setRvisitTime(s.getHfsj());
            o.setNegativeType(s.getBmylx());
            o.setNegativeRemark(s.getBmybeiz());
            o.setNegativeTime(s.getBmysj());
            // 产品大类
            o.setMainId(sysDictRefDLMap.get(String.valueOf(s.getSplb())).get(0).getDictCode());

            o.setAppointmentType(s.getMydlx());
            o.setType(s.getSxlx());
            o.setCreateBy(s.getCreatedBy());
            o.setCreateTime(s.getCreatedDate());
            oanew.add(o);
        }
    }

    private void getOrderAppointment(OrderBase orderBase, List<OrderAppointment> oa, List<ItfTblAzAssignAppointmentDownload> appointments) {
        for (ItfTblAzAssignAppointmentDownload a :appointments){
            OrderAppointment v = new OrderAppointment();
            v.setId(a.getId());
            v.setOrderBaseId(orderBase.getId());
            v.setStartTime(a.getKssj());
            v.setEndTime(a.getJssj());
            v.setType(a.getLeix());
            v.setDelayType(a.getReason());
            v.setRemark(a.getBeiz());
            v.setCreateBy(a.getCzren());
            v.setCreateTime(a.getCzsj());
            oa.add(v);
        }
    }

    private OrderInstall getOrderInstall(OrderBase orderBase, Brand brand, ItfTblAzAssignLcLsDownload item,ItfTblAzAssignFeeDownload fyqd) {
        OrderInstall orderInstall = new OrderInstall();

        if (fyqd !=null){
            OrderPar request = new OrderPar();
            request.setSynReqId(fyqd.getSynReqId());
            request.setBrandId(brand.getBrandId());
            request.setMainSynReqId(item.getSynReqId());

            orderInstall = azGDFYQDDownloadMapper.getOrderFYQD(request);
            // 工单号
            orderInstall.setOrderBaseId(orderBase.getId());
            // 创建人,创建时间
            orderInstall.setCreateBy(orderBase.getCreateBy());
            orderInstall.setCreateTime(orderBase.getCreateTime());
            return orderInstall;
        }
        return null;
    }

    @NotNull
    private List<OrderInstallOperatingLog> getOrderInstallOperatingLogs(List<ItfTblAzAssignFkmxDownload> fkmxs, OrderBase orderBase, Brand brand, ItfTblAzAssignLcLsDownload item) {
        List<OrderInstallOperatingLog> oiolList = new ArrayList<>();
        for (ItfTblAzAssignFkmxDownload fkmx : fkmxs){
            OrderPar orderPar = new OrderPar();
            orderPar.setSynReqId(fkmx.getSynReqId());
            orderPar.setBrandId(brand.getBrandId());
            OrderInstallOperatingLog oiol = azGDFkMxDownloadMapper.getOrderFkMx(orderPar);

            // 工单号码
            oiol.setOrderBaseId(orderBase.getId());
            // 客户名称,客户联系电话
            oiol.setCustomer(orderBase.getUserName());
            oiol.setCustomerPhone(orderBase.getUserMobile());
            oiol.setPgid(orderBase.getPgid());
            oiol.setUploadNotifyFlag(0);
            // websit_number及operate_websit 可能为空，需从销售网点再次查询
            if (StringUtils.isNotEmpty(fkmx.getWdno()) && StringUtils.isEmpty(oiol.getWebsitNumber())){
                // 从销售网点查
                CustomerInformationWebsit ciw = customerInformationWebsitService.lambdaQuery()
                        .eq(CustomerInformationWebsit::getRefWebsitId, fkmx.getWdno())
                        .eq(CustomerInformationWebsit::getBrandId, brand.getBrandId())
                        .one();
                if (ciw !=null){
                    oiol.setWebsitNumber(ciw.getWebsitId());
                    oiol.setOperateWebsit(ciw.getWebsitId());
                }
            }

            // 操作前工单状态,操作前派工状态,操作后工单状态,操作后派工状态 这些字段不处理
            oiolList.add(oiol);
        }
        return oiolList;
    }

    private void getOrderProduct(OrderBase orderBase, List<OrderProduct> orderProducts, Map<String,
            List<SysDictRef>> sysDictRefDLMap, Map<String, List<SysDictRef>> sysDictRefXLMap,
                                 Map<String, List<SysDictRef>> sysDictRefXiLieMap,
                                 List<ItfTblAzAssignMxDownload> orderMx) {

        // pgmxid主键
        // 生成对象
        for (ItfTblAzAssignMxDownload mx:orderMx){
            OrderProduct op = new OrderProduct();
            op.setId(UUID.randomUUID().toString());
            op.setOrderBaseId(orderBase.getId());
            op.setPgmxid(mx.getPgmxid());
            op.setPgguid(mx.getPgguid());
            op.setMainId(sysDictRefDLMap.get(String.valueOf(mx.getSpid())).get(0).getDictCode());
            op.setMainName(sysDictRefDLMap.get(String.valueOf(mx.getSpid())).get(0).getDictValue());
            if (!mx.getXlid().equals(0L)){
                op.setSmallId(sysDictRefXLMap.get(String.valueOf(mx.getXlid())).get(0).getDictCode());
                op.setSmallName(sysDictRefXLMap.get(String.valueOf(mx.getXlid())).get(0).getDictValue());
            } else {
                op.setSmallId(String.valueOf(mx.getXlid()));
                op.setSmallName(mx.getXlmc());
            }

            if (!mx.getXiid().equals("0")){
                op.setSeriesId(sysDictRefXiLieMap.get(mx.getXiid()).get(0).getDictCode());
                op.setSeriesName(sysDictRefXiLieMap.get(mx.getXiid()).get(0).getDictValue());
            } else {
                op.setSeriesId(mx.getXiid());
                op.setSeriesName(mx.getXimc());
            }
            op.setProductId(mx.getJxno());
            op.setProductName(mx.getJxmc());
            op.setInsideCode(mx.getNjtm());
            op.setOutCode(mx.getWjtm());
            op.setNum(mx.getShul());
            op.setPrice(mx.getJiage());
            op.setUnit(mx.getDanw());
            op.setMaterialNumber(mx.getWldm());
            op.setInsideNum(mx.getNjsl());
            op.setOutNum(mx.getWjsl());
            op.setUndoneNum(mx.getWwsl());
            op.setCreateTime(mx.getCjdt());
            op.setCreateBy(mx.getCreatedBy());
            // 销售单位
            op.setSaleWebsit(orderBase.getSaleWebsitId());
            // 无销售单位联系人电话
            op.setServiceWebsit(orderBase.getWebsitId());
            // 无服务单位电话信息 最好取websit表中的热线，先不取
            op.setBuyTime(orderBase.getBuyTime());
            op.setRemark(mx.getBeiz());
            orderProducts.add(op);
        }
    }


    private Map<String, List<SysDictRef>> getMap(Brand brand) {
        List<SysDictRef> sysDictxilieRef;
        Map<String, List<SysDictRef>> sysDictRefXiLieMap;
        sysDictxilieRef =  sysDictRefService.lambdaQuery()
                .select(SysDictRef::getDictCode,SysDictRef::getDictValue,SysDictRef::getRefDictCode)
                .eq(SysDictRef::getBrandId, brand.getBrandId())
                .eq(SysDictRef::getDictType, DictTypeEnum.DICT_TYPE_SERIES.getCode())
                .list();
        sysDictRefXiLieMap = sysDictxilieRef.stream().
                collect(Collectors.groupingBy(SysDictRef::getRefDictCode));
        return sysDictRefXiLieMap;
    }

    private Map<String, List<SysDictRef>> getListMap(Brand brand) {
        List<SysDictRef> sysDictxlRef;
        Map<String, List<SysDictRef>> sysDictRefXLMap;
        sysDictxlRef =  sysDictRefService.lambdaQuery()
                .select(SysDictRef::getDictCode,SysDictRef::getDictValue,SysDictRef::getRefDictCode)
                .eq(SysDictRef::getBrandId, brand.getBrandId())
                .eq(SysDictRef::getDictType, DictTypeEnum.DICT_TYPE_SMALL_TYPE.getCode())
                .list();
        sysDictRefXLMap = sysDictxlRef.stream().
                collect(Collectors.groupingBy(SysDictRef::getRefDictCode));
        return sysDictRefXLMap;
    }

    private Map<String, List<SysDictRef>> getStringListMap(Brand brand) {
        List<SysDictRef> sysDictRef;
        Map<String, List<SysDictRef>> sysDictRefDLMap;
        sysDictRef =  sysDictRefService.lambdaQuery()
                .select(SysDictRef::getDictCode,SysDictRef::getDictValue,SysDictRef::getRefDictCode)
                .eq(SysDictRef::getBrandId, brand.getBrandId())
                .eq(SysDictRef::getDictType, DictTypeEnum.DICT_TYPE_MAIN_TYPE.getCode())
                .list();
        sysDictRefDLMap = sysDictRef.stream().
                collect(Collectors.groupingBy(SysDictRef::getRefDictCode));
        return sysDictRefDLMap;
    }

    private OrderWorker getOrderWorker(OrderBase orderBase) {
        if (StringUtils.isNotEmpty(orderBase.getWorkerName())){
            OrderWorker orderWorker = new OrderWorker();
            orderWorker.setOrderBaseId(orderBase.getId());
            orderWorker.setWorkerId(orderBase.getWorkerId());
            orderWorker.setWorkerName(orderBase.getWorkerName());
            orderWorker.setWorkerMobile(orderBase.getWorkerMobile());
            orderWorker.setWorkerIdcard(orderBase.getWorkerIdcard());
            orderWorker.setIsMaster(Boolean.TRUE);
            orderWorker.setBrandPgid(orderBase.getPgid());
            orderWorker.setPgguid(orderBase.getPgguid());
            orderWorker.setCreateTime(orderBase.getCreateTime());
            orderWorker.setCreateBy(orderBase.getCreateBy());
            orderWorker.setOutWorkerSource(Boolean.FALSE);

            return orderWorker;
        }

        return null;
    }

    private OrderBase getOrderBase(ItfTblAzAssignLcLsDownload item, Brand brand) {
        OrderBase orderBase = new OrderBase();
        OrderPar orderPar = new OrderPar();
        orderPar.setSynReqId(item.getSynReqId());
        orderPar.setBrandId(brand.getBrandId());

        // 生成工单基础表 对象
        orderBase = azGDLcLsDownloadMapper.getOrderBase(orderPar);
        // 总部下来的工单新增时统一为待接收状态
        orderBase.setDispatchStatus("DJS");
        //  预派工订单标识
        if (orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_A1100.getCode()) ||
                orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_A1122.getCode()) ||
                orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_A1130.getCode()) ){
            orderBase.setIsReservation(1);
        }

        // 重置部分数据
        orderService.setPar(item, orderBase, brand);

        return orderBase;
    }

}
