package com.zfire.jiasm.syncdata.download.order.repair;

import com.zfire.jiasm.syncdata.base.BaseService;
import com.zfire.jiasm.syncdata.dao.mapper.AssignDownloadMapper;
import com.zfire.jiasm.syncdata.dao.mapper.DetailOrderRepairMapper;
import com.zfire.jiasm.syncdata.dao.mapper.RepairDownloadMapper;
import com.zfire.jiasm.syncdata.parameter.OrderPar;
import com.zfire.jiasm.syncdata.plus.entity.Brand;
import com.zfire.jiasm.syncdata.plus.entity.ItfTblAssignDownload;
import com.zfire.jiasm.syncdata.plus.entity.OrderApplyDispatch;
import com.zfire.jiasm.syncdata.plus.entity.OrderBase;
import com.zfire.jiasm.syncdata.plus.service.ItfTblAssignDownloadService;
import com.zfire.jiasm.syncdata.plus.service.OrderApplyDispatchService;
import com.zfire.jiasm.syncdata.plus.service.OrderBaseService;
import com.zfire.jiasm.syncdata.service.OrderService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 格力品牌维修工单下载后落本地数据表
 * 1：落业务表
 * 2：修改下载主表处理结果
 */
@Slf4j
@Component
public class RepairBusiness {

    @Resource
    RepairBusiness repairBusiness;
    @Resource
    ItfTblAssignDownloadService itfTblAssignDownloadService;
    @Resource
    RepairDownloadMapper repairDownloadMapper;
    @Resource
    BaseService baseService;
    @Resource
    OrderBaseService orderBaseService;
    @Resource
    OrderService orderService;
    @Resource
    DetailOrderRepairMapper detailOrderRepairMapper;
    @Resource
    OrderApplyDispatchService orderApplyDispatchService;
    @Resource
    AssignDownloadMapper assignDownloadMapper;


    // 1：工单落入本地表
    @Scheduled(fixedDelay = 1 * 10 * 1000)
    public void inputOrder() {
        /*if (1 == 1) {
            return;
        }*/


        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<ItfTblAssignDownload> repairRec = new ArrayList<>();

        // 提取品牌信息
        Brand brand = baseService.getBrand();
        if (brand == null) {
            return;
        }

        // 提取待落本地表的数据
        repairRec = itfTblAssignDownloadService.lambdaQuery()
                .eq(ItfTblAssignDownload::getSynStatus,0)
                .orderByAsc(ItfTblAssignDownload::getSynReqId)
                .list();

        if (repairRec.size()<1){
            log.info("暂无待处理的维修工单: {}",sdf.format(new Date()));
            return;
        }

        log.info("维修工单落本地表开始处理: {}",sdf.format(new Date()));

        // 按任务号download_task_no，按单来处理
        for (ItfTblAssignDownload item: repairRec){

            // 新增
            repairBusiness.addOrder(brand, item);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public  void addOrder(Brand brand, ItfTblAssignDownload item) {
        // 新增 TL_SynDataFrom_tbl_assign_batch
        // TL_SynDataFrom_tbl_assign 主单的 syn_req_id
        // 生成维修工单主表数据
        OrderBase order = saveOrderBase(brand, item);

        // 更新 修改时更新 新增时无需此操作 可以先放进来，是修改不到数据的
        // 维修工单产品信息 (维修网点、维修网点名称) websit_id，websit_name
        // order_product 无维修网点信息，所以不用做

        // 采集表需要做上面的操作 order_repair_detail  新增时也无需操作，加上也不会出错
        OrderPar request = new OrderPar();
        request.setOrderNo(order.getId());
        detailOrderRepairMapper.updateWebsitMessage(request);


        // 新增或是修改都要加  新增是一定要做的
        // 总部维修单为待审批改派时 若本地表无改派数据则生成改派数据
        if (order.getOrderStatus().equals("B5")){
            generateDispatch(order);
        }

        // 原 本地维修单表状态为5 待审批  同步过来的工单状态不是待审批时
        // 修改接品自动审核  新增时条件不成立的

        // 增加判断是否有未处理的待改派，如果广佛有待改派但总部工单不是待改派，直接更新改派标志
        // 新增时了样不用操作

        // 增加处理采集数据下载 新增不用操作
            /*if ((order.getOrderStatus().equals("B41") || order.getOrderStatus().equals("B102"))){

            }*/
        // 如果是家用的工单，且工单状态是待中心派工或待网点派工时，写入派嘉讯茂  不做

        // 修改同步状态
        upItfTblAssignDownloadStatus(item, order);
    }

    private OrderBase saveOrderBase(Brand brand, ItfTblAssignDownload item) {
        // 获取维修工单主表数据
        OrderPar orderPar = new OrderPar();
        orderPar.setSynReqId(item.getSynReqId());
        orderPar.setBrandId(brand.getBrandId());
        OrderBase order = repairDownloadMapper.getOrderBase(orderPar);
        // 部分属性值重置
        orderService.setParame(brand, order);
        // 新增
        orderBaseService.save(order);
        return order;
    }

    public void upItfTblAssignDownloadStatus(ItfTblAssignDownload item, OrderBase order){
        ItfTblAssignDownload up = new ItfTblAssignDownload();
        up.setSynReqId(item.getSynReqId());
        up.setSynStatus(1);
        up.setSynTime(new Date());
        up.setSynErrMsg("成功");
        itfTblAssignDownloadService.updateById(up);

        OrderPar prepar = new OrderPar();
        prepar.setSynReqId(item.getSynReqId());
        prepar.setPgid(order.getPgid());
        assignDownloadMapper.updateStatus(prepar);
    }

    private void generateDispatch(OrderBase order) {
        List<String> parame = new ArrayList<>();
        parame.add(order.getWebsitId());
        parame.add(order.getReceWebsitId());
        // 改派表中查看是否有待审的数据
        List<OrderApplyDispatch> rec = orderApplyDispatchService.lambdaQuery()
                .eq(OrderApplyDispatch::getOrderBaseId, order.getId())
                .eq(OrderApplyDispatch::getExamineStatus,"WAIT")
                .in(OrderApplyDispatch::getWebsitId,parame).list();
        if (rec.size()<1){
            // 生成改派数据 从维修工单表中读取数据
            OrderApplyDispatch addn = new OrderApplyDispatch();
            addn.setOrderBaseId(order.getId());
            addn.setPgid(order.getPgid());
            addn.setType("REPAIR");
            addn.setWebsitId(order.getWebsitId());
            addn.setWebsitName(order.getWebsitName());
            addn.setApplyContent("申请改派");
            addn.setExamineStatus("WAIT");
            addn.setUserName(order.getUserName());
            addn.setUserMobile(order.getUserMobile());
            addn.setReserveTime(order.getAppointmentTime());
            addn.setReservePeriod(String.valueOf(order.getAppointmentTime()));
            addn.setMainId(Integer.valueOf(order.getMainId()));
            addn.setMainName(order.getMainName());
            addn.setCreateTime(order.getCreateTime());
            addn.setUpdateTime(order.getUpdateTime());
            orderApplyDispatchService.save(addn);
        }
    }


}
