package com.zfire.jiasm.syncdata.service.impl;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zfire.jiasm.syncdata.plus.entity.*;
import com.zfire.jiasm.syncdata.plus.mapper.OrderBaseMapper;
import com.zfire.jiasm.syncdata.plus.service.*;
import com.zfire.jiasm.syncdata.service.OrderService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Service
public class OrderServiceImpl extends ServiceImpl<OrderBaseMapper, OrderBase> implements OrderService {
    @Resource
    CustomerInformationWebsitService customerInformationWebsitService;
    @Resource
    WebsitOrderSignService websitOrderSignService;
    @Resource
    StreetService streetService;
    @Resource
    BrandLbsRelateService brandLbsRelateService;
    @Resource
    AreaService areaService;


    @Override
    public  void setPar(ItfTblAzAssignLcLsDownload item, OrderBase orderBase, Brand brand){
        // 品牌信息
        orderBase.setBrandId(brand.getBrandId());
        orderBase.setBrandName(brand.getBrand());

        // 判断安装网点编号是否为空或者为公司，如果为空或为公司，就写入接收派工网点
        if (StringUtils.isEmpty(item.getAzwdno()) || item.getAzwdno().equals("S9219801") || item.getAzwdno().equals("S9219807")){
            if (orderBase.getReceWebsitId() !=null){
                orderBase.setWebsitId(orderBase.getReceWebsitId());
                orderBase.setWebsitName(orderBase.getReceWebsitName());
            }
        }

        // 省市区单独处理
        getPCAS(orderBase);
        // 派工方式，新创建订单时 azren 名称不为空时为手动派工  （dispatch_flag派工方式 A=自动派工 B=手动派工）
        if (StringUtils.isNotEmpty(orderBase.getDispatchFlag())){
            orderBase.setDispatchFlag("B");
        } else {
            orderBase.setDispatchFlag("A");
        }

        // create_sys - 创建网点系统  cjwdno
        // last_operator_websit:最近操作网点 创建单时也是创建网点 可能是销售网点，也可能是售后网点  zjczwd
        // sys_websit_number 系统网点编号 zjczwd 同上

        // 先从销售网点查
        CustomerInformationWebsit ciw = customerInformationWebsitService.lambdaQuery()
                .eq(CustomerInformationWebsit::getRefWebsitId, orderBase.getCreateSys())
                .eq(CustomerInformationWebsit::getBrandId, brand.getBrandId())
                .one();
        if (ciw !=null){
            orderBase.setCreateSys(ciw.getWebsitId());
        } else {
            WebsitOrderSign wos = websitOrderSignService.lambdaQuery()
                    .eq(WebsitOrderSign::getBrandWebsitId, orderBase.getCreateSys())
                    .eq(WebsitOrderSign::getBrandId, brand.getBrandId())
                    .one();
            if (wos != null){
                orderBase.setCreateSys(wos.getWebsitId());
            } else {
                orderBase.setCreateSys("");
            }
        }

        // 先从销售网点查
        CustomerInformationWebsit ciwx = customerInformationWebsitService.lambdaQuery()
                .eq(CustomerInformationWebsit::getRefWebsitId, orderBase.getLastOperatorWebsit())
                .eq(CustomerInformationWebsit::getBrandId, brand.getBrandId())
                .one();
        if (ciwx !=null){
            orderBase.setLastOperatorWebsit(ciwx.getWebsitId());
            orderBase.setSysWebsitNumber(ciwx.getWebsitId());
        } else {
            WebsitOrderSign wos = websitOrderSignService.lambdaQuery()
                    .eq(WebsitOrderSign::getBrandWebsitId, orderBase.getLastOperatorWebsit())
                    .eq(WebsitOrderSign::getBrandId, brand.getBrandId())
                    .one();
            if (wos != null){
                orderBase.setLastOperatorWebsit(wos.getWebsitId());
                orderBase.setSysWebsitNumber(wos.getWebsitId());
            } else {
                orderBase.setLastOperatorWebsit("");
                orderBase.setSysWebsitNumber("");
            }
        }
    }



    @Override
    public  void setParame(Brand brand, OrderBase orderBase) {
        orderBase.setBrandId(brand.getBrandId());
        orderBase.setBrandName(brand.getBrand());
        // 总部下来的工单统一为待接收状态-派工状态
        orderBase.setDispatchStatus("DJS");

        // 省市区单独处理
        getPCAS(orderBase);

        // 派工方式，新创建订单时 azren 名称不为空时为手动派工  （dispatch_flag派工方式 A=自动派工 B=手动派工）
        if (StringUtils.isNotEmpty(orderBase.getDispatchFlag())){
            orderBase.setDispatchFlag("B");
        } else {
            orderBase.setDispatchFlag("A");
        }

        // create_sys - 创建网点系统  cjwdno
        // last_operator_websit:最近操作网点 创建单时也是创建网点 可能是销售网点，也可能是售后网点  zjczwd
        // sys_websit_number 系统网点编号 zjczwd 同上
        // 先从销售网点查
        CustomerInformationWebsit ciw = customerInformationWebsitService.lambdaQuery()
                .eq(CustomerInformationWebsit::getRefWebsitId, orderBase.getCreateSys())
                .eq(CustomerInformationWebsit::getBrandId, brand.getBrandId())
                .one();
        if (ciw !=null){
            orderBase.setCreateSys(ciw.getWebsitId());
        } else {
            WebsitOrderSign wos = websitOrderSignService.lambdaQuery()
                    .eq(WebsitOrderSign::getBrandWebsitId, orderBase.getCreateSys())
                    .eq(WebsitOrderSign::getBrandId, brand.getBrandId())
                    .one();
            if (wos != null){
                orderBase.setCreateSys(wos.getWebsitId());
            } else {
                orderBase.setCreateSys("");
            }
        }

        // 先从销售网点查
        CustomerInformationWebsit ciwx = customerInformationWebsitService.lambdaQuery()
                .eq(CustomerInformationWebsit::getRefWebsitId, orderBase.getLastOperatorWebsit())
                .eq(CustomerInformationWebsit::getBrandId, brand.getBrandId())
                .one();
        if (ciwx !=null){
            orderBase.setLastOperatorWebsit(ciwx.getWebsitId());
            orderBase.setSysWebsitNumber(ciwx.getWebsitId());
        } else {
            WebsitOrderSign wos = websitOrderSignService.lambdaQuery()
                    .eq(WebsitOrderSign::getBrandWebsitId, orderBase.getLastOperatorWebsit())
                    .eq(WebsitOrderSign::getBrandId, brand.getBrandId())
                    .one();
            if (wos != null){
                orderBase.setLastOperatorWebsit(wos.getWebsitId());
                orderBase.setSysWebsitNumber(wos.getWebsitId());
            } else {
                orderBase.setLastOperatorWebsit("");
                orderBase.setSysWebsitNumber("");
            }
        }
    }

    private void getPCAS(OrderBase orderBase) {
        List<BrandLbsRelate> br;
        if (StringUtils.isNotEmpty(orderBase.getStreet())){
            Street street = streetService.lambdaQuery()
                    .eq(Street::getProvince, orderBase.getProvince())
                    .eq(Street::getCity, orderBase.getCity())
                    .eq(Street::getArea, orderBase.getArea())
                    .eq(Street::getStreet, orderBase.getStreet()).one();
            getPCA(orderBase, street);
            //街道
            br = brandLbsRelateService.lambdaQuery()
                    .eq(BrandLbsRelate::getStreetId,street.getStreetId())
                    .isNotNull(BrandLbsRelate::getLbsDistrictId).list();
            if (br !=null && br.size()>0){
                orderBase.setStreetId(br.get(0).getLbsStreetId());
                orderBase.setStreet(br.get(0).getLbsStreetName());
            }
        } else {
            // 总部区必选
            Area area = areaService.lambdaQuery()
                    .eq(Area::getArea, orderBase.getArea()).one();
            Street st = new Street();
            if (area !=null){
                BeanUtils.copyProperties(area,st);
                getPCA(orderBase, st);
                orderBase.setStreet("");
            } else {
                orderBase.setProvinceId("");
                orderBase.setCity("");
                orderBase.setArea("");
                orderBase.setStreet("");
            }
        }
    }

    private void getPCA(OrderBase orderBase, Street street) {
        List<BrandLbsRelate> br;
        //省
        br = brandLbsRelateService.lambdaQuery()
                .eq(BrandLbsRelate::getProvinceId, street.getProvinceId())
                .isNotNull(BrandLbsRelate::getLbsProvinceId).list();
        if (br !=null && br.size()>0){
            orderBase.setProvinceId(br.get(0).getLbsProvinceId());
            orderBase.setProvince(br.get(0).getLbsProvinceName());
        }
        //市
        br = brandLbsRelateService.lambdaQuery()
                .eq(BrandLbsRelate::getCityId, street.getCityId())
                .isNotNull(BrandLbsRelate::getLbsCityId).list();
        if (br !=null && br.size()>0){
            orderBase.setCityId(br.get(0).getLbsCityId());
            orderBase.setProvince(br.get(0).getLbsCityName());
        }
        //区
        br = brandLbsRelateService.lambdaQuery()
                .eq(BrandLbsRelate::getAreaId, street.getAreaId())
                .isNotNull(BrandLbsRelate::getLbsDistrictId).list();
        if (br !=null && br.size()>0){
            orderBase.setAreaId(br.get(0).getLbsDistrictId());
            orderBase.setArea(br.get(0).getLbsDistrictName());
        }
    }

}
