package com.zfire.jiasm.syncdata.upload.order;

import com.zfire.jiasm.syncdata.constant.OrderStatusEnum;
import com.zfire.jiasm.syncdata.plus.entity.*;
import com.zfire.jiasm.syncdata.plus.service.*;
import com.zfire.jiasm.syncdata.service.PrepareService;
import com.zfire.jiasm.syncdata.utils.DataUtil;
import com.zfire.jiasm.syncdata.utils.PrepareTaskLog;
import com.zfire.jiasm.syncdata.utils.TokenCenter;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * 生成上传任务清单列表
 * 1:从需要上传的订单数据表中提取数据 out_jiasm_to_gree_uploadnotifylist
 *:2:数据同步预处理规则表中提取对应表名的规则 out_jiasm_to_gree_prepare_rule
 * 3:对应表中提取表中数据
 * 4：生成上传任务清单表 out_jiasm_to_gree_upload_task_list
 */
@Slf4j
@Component
public class ProduceUploadList {
    @Value("${baseurl}")
    private String baseurl = "";

    @Resource
    ProduceUploadList produceUploadList;
    @Resource
    TokenCenter tokenCenter;
    @Resource
    PrepareService prepareService;
    @Resource
    OutJiasmToGreeUploadTaskListService outJiasmToGreeUploadTaskListService;
    @Resource
    OrderBaseService orderBaseService;
    @Resource
    OutJiasmToGreeUploadnotifylistService outJiasmToGreeUploadnotifylistService;
    @Resource
    OrderInstallOperatingLogService orderInstallOperatingLogService;
    @Resource
    OrderReAppointmentService orderReAppointmentService;





    @Scheduled(fixedDelay = 1 * 20 * 1000)
    private void produceList(){
        if (1==1){
            return;
        }

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        List<OutJiasmToGreeUploadTaskList> ojtgutls = new ArrayList<>();

        // Token token = tokenCenter.getCurrentToken();

        log.info("开始预处理需要上传的订单数据: {}",sdf.format(new Date()));

        // 从需要上传的订单数据表中提取数据
        List<OutJiasmToGreeUploadnotifylist> taskResults = outJiasmToGreeUploadnotifylistService.lambdaQuery()
                .eq(OutJiasmToGreeUploadnotifylist::getSynStatus,0)
                .orderByAsc(OutJiasmToGreeUploadnotifylist::getSynReqId).list();

        try {
            for (OutJiasmToGreeUploadnotifylist item : taskResults) {
                Long itemSynReqId = item.getSynReqId();
                // 同步的表名
                String itemTableName = item.getTableName();
                // 同步表中的工单号
                String itemOrderNo = item.getOrderNo();
                // 同步表中的主键id
                String itemTableNameId = item.getId();
                String itemOrderType = item.getOrderType();

                // 数据同步预处理规则表中提取对应表名的规则
                Map<String, Object> row = prepareService.getTaskInfo(itemTableName);
                int taskId = (Integer) row.get("id");
                String srcTabName = (String) row.get("src_tab_name");

                String errorMessage = null;
                try {
                    // 根据表名获取表中指定工单的数据
                    // 规则表中表名，同步表中的单号，同步表中的id(同步表中的表名的主键)
                    // Map<String, Object> sourceData = prepareService.getTaskDataDetail(srcTabName, orderNo,topOrderId);

                    // 过程名，规则表中的ID，源表中的数据最后修改时间，需同步的表中的工单号，需同步的表中的主键id,空值，false
                    // prepareService.prepareData(procName, taskId, date, sOrderNo, orderId, newDykh, isNeedGenRemoteId);
                    // 工单表执行 TL_PrePare_task_by_fa_order_install 过程

                    // 数据上传任务表OutJiasmToGreeUploadTaskList 生成数据

                    // 安装工单处理
                    if (itemTableName.equals("order_base") && itemOrderType.equals("INSTALL")){
                        produceUploadList.orderBaseProess(itemTableNameId, ojtgutls, item.getSynReqId(), itemTableName, itemOrderType);
                    }
                    // 安装工单操作明细
                    if (itemTableName.equals("order_install_operating_log")){
                        produceUploadList.orderInstallOperatingLog(item, itemTableName, itemOrderType);
                    }
                    // 安装工单用户改约明细
                    if (itemTableName.equals("order_re_appointment")){
                        produceUploadList.OrderReAppointment(item, itemTableName, itemOrderType);
                    }




                } catch (Exception ex) {
                    errorMessage = ex.getMessage();
                    updateError(errorMessage, item);
                } finally {
                    PrepareTaskLog.writeMessage(genInfoData(taskId, srcTabName, false, errorMessage));
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            log.info("开始预处理需要上传的订单数据出错");
        }
    }

    @Transactional(rollbackFor = Exception.class)
    public  void OrderReAppointment(OutJiasmToGreeUploadnotifylist item, String itemTableName, String itemOrderType) {
        OrderReAppointment rea = orderReAppointmentService.lambdaQuery()
                .eq(OrderReAppointment::getId, item.getId()).one();
        // 同步任务增加一条数据
        if (rea !=null){
            OutJiasmToGreeUploadTaskList add = getOutJiasmToGreeUploadTaskList(item.getOrderNo(), item.getId(),
                    rea.getCreateTime(), itemTableName, itemOrderType, 230);
            outJiasmToGreeUploadTaskListService.save(add);
        }

        // 需要上传的订单数据表中处理状态同步为已处理
        OutJiasmToGreeUploadnotifylist up = getOutJiasmToGreeUploadnotifylist(item.getSynReqId());
        outJiasmToGreeUploadnotifylistService.updateById(up);
    }

    @Transactional(rollbackFor = Exception.class)
    public void orderInstallOperatingLog(OutJiasmToGreeUploadnotifylist item, String itemTableName, String itemOrderType) {
        OrderInstallOperatingLog logInfo = orderInstallOperatingLogService.lambdaQuery()
                .eq(OrderInstallOperatingLog::getId, item.getId()).one();
        // 同步任务增加一条数据
        if (logInfo !=null){
            OutJiasmToGreeUploadTaskList add = getOutJiasmToGreeUploadTaskList(item.getOrderNo(), item.getId(),
                    logInfo.getCreateTime(), itemTableName, itemOrderType, 23);
            outJiasmToGreeUploadTaskListService.save(add);
        }

        // 需要上传的订单数据表中处理状态同步为已处理
        OutJiasmToGreeUploadnotifylist up = getOutJiasmToGreeUploadnotifylist(item.getSynReqId());
        outJiasmToGreeUploadnotifylistService.updateById(up);
    }


    public void updateError(String errMessage, OutJiasmToGreeUploadnotifylist item) {
        if (errMessage != null && errMessage.trim().length() > 255) {
            errMessage = errMessage.substring(0, 255);
        }
        OutJiasmToGreeUploadnotifylist up = new OutJiasmToGreeUploadnotifylist();
        up.setSynReqId(item.getSynReqId());
        up.setSynTimes(item.getSynTimes()+1);
        up.setSynErrMsg(errMessage);
        up.setSynReqTime(new Date());
        outJiasmToGreeUploadnotifylistService.updateById(up);
    }

    @NotNull
    private OutJiasmToGreeUploadnotifylist getOutJiasmToGreeUploadnotifylist(Long synReqId) {
        OutJiasmToGreeUploadnotifylist up = new OutJiasmToGreeUploadnotifylist();
        up.setSynReqId(synReqId);
        up.setSynStatus(1);
        up.setSynReqTime(new Date());
        return up;
    }


    @Transactional(rollbackFor = Exception.class)
    public void orderBaseProess(String itemTableNameId, List<OutJiasmToGreeUploadTaskList> ojtgutls, Long synReqId, String itemTableName, String itemOrderType) {
        OrderBase orderBase = orderBaseService.lambdaQuery()
                .eq(OrderBase::getId,itemTableNameId).one();

        // 安装工单处理
        if (orderBase !=null){
            InstallOrderBase(itemTableNameId, orderBase.getUpdateTime(), orderBase, ojtgutls, itemTableName, itemOrderType);
        }

        if (ojtgutls.size()>0){
            outJiasmToGreeUploadTaskListService.saveBatch(ojtgutls);
            ojtgutls.clear();
        }
        // 需要上传的订单数据表中处理状态同步为已处理
        OutJiasmToGreeUploadnotifylist up = getOutJiasmToGreeUploadnotifylist(synReqId);
        outJiasmToGreeUploadnotifylistService.updateById(up);
    }

    private void InstallOrderBase(String orderNo, Date updateTime, OrderBase orderBase, List<OutJiasmToGreeUploadTaskList> ojtgutls,
                                  String itemTableName, String itemOrderType) {

        // 若工单被删除不再同步 不写上传记录表

        // 生成一条22类型的上传记录  out_jiasm_to_gree_upload_task_list
        OutJiasmToGreeUploadTaskList add = getOutJiasmToGreeUploadTaskList(orderNo, orderBase.getId(), updateTime, itemTableName, itemOrderType, 22);
        ojtgutls.add(add);
        // 工单状态order_status 6, 7, 9, 10
        if (orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I6.getCode()) ||
                orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I7.getCode()) ||
                orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I9.getCode()) ||
                orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I10.getCode())){
            if (orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I6.getCode())){
                OutJiasmToGreeUploadTaskList addo = getOutJiasmToGreeUploadTaskList(orderNo, orderBase.getId(), updateTime, itemTableName, itemOrderType, 221);
                ojtgutls.add(addo);
            } else if (orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I7.getCode()) ||
                    orderBase.getOrderStatus().equals(OrderStatusEnum.INSTALL_STATUS_I9.getCode())){
                OutJiasmToGreeUploadTaskList addt = getOutJiasmToGreeUploadTaskList(orderNo, orderBase.getId(), updateTime, itemTableName, itemOrderType, 221);
                ojtgutls.add(addt);
                OutJiasmToGreeUploadTaskList addtt = getOutJiasmToGreeUploadTaskList(orderNo, orderBase.getId(), orderBase.getUpdateTime(), itemTableName, itemOrderType, 222);
                ojtgutls.add(addtt);
            } else {
                OutJiasmToGreeUploadTaskList addttt = getOutJiasmToGreeUploadTaskList(orderNo, orderBase.getId(), updateTime, itemTableName, itemOrderType, 222);
                ojtgutls.add(addttt);
            }
        }
    }

    @NotNull
    private OutJiasmToGreeUploadTaskList getOutJiasmToGreeUploadTaskList(String orderNo, String id, Date updateTime, String itemTableName, String itemOrderType, int i) {
        OutJiasmToGreeUploadTaskList add = new OutJiasmToGreeUploadTaskList();
        add.setUploadTaskType(i);
        add.setOrderNo(orderNo);
        add.setId(id);
        add.setTableName(itemTableName);
        add.setOrderType(itemOrderType);
        add.setUpdatetime(updateTime);
        return add;
    }

    private String genInfoData(int taskId, String srcTabName, boolean isTimeOut, String errorMessage) {
        String result = "预处理任务[" + taskId + "," + srcTabName + "] " + (isTimeOut ? "超时!!!!" : "正常") + " "
                + (errorMessage != null ? "错误:[" + errorMessage + "]" : "");
        return result;
    }

    private Map<String, Object> genDataMapFromSource(String localKeyList, String serverKeyList,
                                                     Map<String, Object> sourceData) {
        Map<String, Object> paramMap = new HashMap<String, Object>();
        String[] localKeys = localKeyList.split("\\Q,\\E");
        String[] serverKeys = serverKeyList.split("\\Q,\\E");
        for (int i = 0; i < localKeys.length; i++) {
            String localKey = localKeys[i];
            String serverKey = serverKeys[i];
            Object value = sourceData.get(localKey.trim());
            paramMap.put(DataUtil.fromDbName2ServiceName(serverKey.trim()), value);
        }

        return paramMap;
    }

}
